#!/usr/bin/env perl

# LSG::Markdown - markdown preprocessor for the LSG
# Written by lumidify <nobody@lumidify.org>
# Last updated: 2019-08-21
#
# To the extent possible under law, the author has dedicated
# all copyright and related and neighboring rights to this
# software to the public domain worldwide. This software is
# distributed without any warranty.
#
# You should have received a copy of the CC0 Public Domain
# Dedication along with this software. If not, see
# <http://creativecommons.org/publicdomain/zero/1.0/>.

package LSG::Markdown;
use strict;
use warnings;
use utf8;
use open qw< :encoding(UTF-8) >;
use File::Spec::Functions;
use Text::Markdown qw(markdown);
use LSG::Misc;
use LSG::Config qw($config);

sub handle_fnc {
	my $pageid = shift;
	my $lang = shift;
	my $line = shift;
	my $file = shift;
	my $fnc_name = shift;
	my @fnc_args = split(/ /, shift);
	if (!exists($config->{"funcs"}->{$fnc_name})) {
		die "ERROR: $file: undefined function \"$fnc_name\":\n$line\n";
	}
	return $config->{"funcs"}->{$fnc_name}->($pageid, $lang, @fnc_args);
}

sub handle_lnk {
	my $pageid = shift;
	my $lang = shift;
	my $line = shift;
	my $file = shift;
	my $txt = shift;
	my $lnk = shift;
	my $lnk_file = "";
	my $lnk_path = "";
	my $url = "";

	my $char_one = substr($lnk, 0, 1);
	if ($char_one eq "@") {
		$lnk_file = $config->{"metadata"}->{$pageid}->{"basename"} . substr($lnk, 1);
		$lnk_path = catfile("site", "static", $lnk_file);
		$url = LSG::Misc::gen_relative_link("$lang/$pageid", "static/$lnk_file");
	} elsif ($char_one eq "#") {
		$lnk_file = substr($lnk, 1);
		$lnk_path = catfile("site", "static", $lnk_file);
		$url = LSG::Misc::gen_relative_link("$lang/$pageid", "static/$lnk_file");
	} elsif ($char_one eq "\$") {
		$lnk_file = substr($lnk, 1);
		$lnk_path = catfile("pages", $lnk_file);
		# Convert to /lang/page format
		my $lnk_abs = substr($lnk_file, -2) . "/" . substr($lnk_file, 0, length($lnk_file) - 3) . ".html";
		$url = LSG::Misc::gen_relative_link("$lang/$pageid", $lnk_abs);
	} else {
		$url = $lnk;
	}
        if ($lnk_path && !(-f $lnk_path)) {
                die "ERROR: $file: linked file $lnk_path does not exist:\n$line\n";
        }
	return "[$txt]($url)";
}

sub handle_img {
        my $pageid = shift;
        my $lang = shift;
        my $line = shift;
        my $file = shift;
        my $txt = shift;
	my $img = shift;
	my $img_file = "";
	my $img_path = "";
	my $src = "";

	my $char_one = substr($img, 0, 1);
        if ($char_one eq "@") {
                $img_file = $config->{"metadata"}->{$pageid}->{"basename"} . substr($img, 1);
		$img_path = catfile("site", "static", $img_file);
		$src = LSG::Misc::gen_relative_link("$lang/$pageid", "static/$img_file");
        } elsif ($char_one eq "#") {
                $img_file = substr($img, 1);
		$img_path = catfile("site", "static", $img_file);
		$src = LSG::Misc::gen_relative_link("$lang/$pageid", "static/$img_file");
        } else {
		$src = $img;
        }
	if ($img_path && !(-f $img_path)) {
		die "ERROR: $file: image file $img_path does not exist:\n$line\n";
	}

	return "![$txt]($src)";
}

sub add_child {
	my $parent = shift;
	my $type = shift;
	$parent->{"child"} = {type => $type, txt => "", url => "", parent => $parent, child => {}};

	return $parent->{"child"};
}

sub finish_child {
	my $child = shift;
	my $pageid = shift;
	my $lang = shift;
	my $line = shift;
	my $file = shift;
	my $parent = $child->{"parent"};

	if ($child->{"type"} eq "img") {
		$parent->{"txt"} .= handle_img($pageid, $lang, $line, $file, $child->{"txt"}, $child->{"url"});
	} elsif ($child->{"type"} eq "lnk") {
		$parent->{"txt"} .= handle_lnk($pageid, $lang, $line, $file, $child->{"txt"}, $child->{"url"});
	} elsif ($child->{"type"} eq "fnc") {
		$parent->{"txt"} .= handle_fnc($pageid, $lang, $line, $file, $child->{"txt"}, $child->{"url"});
	}

	return $parent;
}

sub parse_md {
	my $lang = shift;
	my $pageid = shift;
	my $inpath = shift;
	open(my $in, "<", $inpath) or die "ERROR: Can't open $inpath for reading.";
	# skip metadata
	while (<$in> =~ /^([^:]*):(.*)$/) {}

	my $txt = "";
	my $bs = 0;
	my $IN_IMG = 1;
	my $IN_LNK = 2;
	my $IN_FNC = 4;
	my $IN_TXT = 8;
	my $IN_URL = 16;
	my $IN_IMG_START = 32;
	my %structure = (txt => "", child => {});
	my $cur_child_ref = \%structure;
	my @states = (0);
	foreach (<$in>) {
		foreach my $char (split //, $_) {
			if ($char eq "\\") {
				$bs++;
				if (!($bs %= 2)) {$txt .= "\\"};
			} elsif ($bs % 2) {
				# FIXME: CLEANUP!!!
				if ($states[-1] & $IN_TXT) {
					$cur_child_ref->{"txt"} .= $char;
				} elsif ($states[-1] & $IN_URL) {
					$cur_child_ref->{"url"} .= $char;
				} elsif (!($states[-1] & ($IN_IMG | $IN_LNK | $IN_FNC))) {
					$structure{"txt"} .= $char;
				}
				$bs = 0;
			} elsif ($char eq "!") {
				push(@states, $IN_IMG_START);
			} elsif ($char eq "[") {
				if ($states[-1] & $IN_IMG_START) {
					$states[-1] = $IN_IMG | $IN_TXT;
					$cur_child_ref = add_child($cur_child_ref, "img");
				} else {
					push(@states, $IN_LNK | $IN_TXT);
					$cur_child_ref = add_child($cur_child_ref, "lnk");
				}
			} elsif ($char eq "{") {
				$cur_child_ref = add_child($cur_child_ref, "fnc");
				push(@states, $IN_FNC | $IN_TXT);
			} elsif ($char eq "]" && ($states[-1] & ($IN_IMG | $IN_LNK) && $states[-1] & $IN_TXT)) {
				$states[-1] &= ~$IN_TXT;
			} elsif ($char eq "}" && $states[-1] & $IN_FNC && $states[-1] & $IN_TXT) {
				$states[-1] &= ~$IN_TXT;
			} elsif ($char eq "(" && $states[-1] & ($IN_IMG | $IN_LNK | $IN_FNC)) {
				$states[-1] |= $IN_URL;
			} elsif ($char eq ")" && ($states[-1] & $IN_URL)) {
				pop(@states);
				$cur_child_ref = finish_child($cur_child_ref, $pageid, $lang, $_, $inpath);
			} else {
				if ($states[-1] & $IN_IMG_START) {pop(@states)}
				if ($states[-1] & $IN_TXT) {
					$cur_child_ref->{"txt"} .= $char;
				} elsif ($states[-1] & $IN_URL) {
					$cur_child_ref->{"url"} .= $char;
				} elsif (!($states[-1] & ($IN_IMG | $IN_LNK | $IN_FNC))) {
					$structure{"txt"} .= $char;
				}
			}
		}
	}

	return markdown($structure{"txt"});
}

1;
