#!/usr/bin/env perl

# LSG::Metadata - metadata parser for the LSG
# Written by lumidify <nobody@lumidify.org>
# Last updated: 2019-08-21
#
# To the extent possible under law, the author has dedicated
# all copyright and related and neighboring rights to this
# software to the public domain worldwide. This software is
# distributed without any warranty.
#
# You should have received a copy of the CC0 Public Domain
# Dedication along with this software. If not, see
# <http://creativecommons.org/publicdomain/zero/1.0/>.

package LSG::Metadata;
use strict;
use warnings;
use utf8;
use open qw< :encoding(UTF-8) >;
use File::Find;
use File::Spec::Functions qw(catfile catdir splitdir);
use File::Path;
use LSG::Config qw($config);

sub parse_metadata_file {
	my $in = shift;
	my %tmp_fm = ();
	while (<$in> =~ /^([^:]*):(.*)$/) {
		$tmp_fm{$1} = $2;
		if (eof) {last}
	}
	return \%tmp_fm;
}

sub parse_metadata {
	if (!(-f)) {return};
	my $fullname = $File::Find::name;
	# Strip "pages/" from dirname
	my @dirs = splitdir($File::Find::dir);
	my $dirname = catdir(@dirs[1..$#dirs]);
	my $basename = substr($_, 0, $#_-2);
	# Note: this will only work if language codes are two chars
	my $lang = substr($_, -2);
	my $pageid = $basename;
	if ($dirname) {$pageid = catfile($dirname, $basename)};
	open(my $in, "<", $_) or die "Can't open $fullname: $!";
	my %tmp_md = %{parse_metadata_file($in)};
	close($in);
	my $modified_date = (stat($_))[9];
	$config->{"metadata"}->{$pageid}->{"modified"}->{$lang} = $modified_date;
	if (!exists($tmp_md{"template"})) {
		die "ERROR: $fullname does not specify a template\n";
	}
	if (!exists($config->{"templates"}->{$tmp_md{"template"} . ".$lang.html"})) {
		die "ERROR: $fullname: template " . $tmp_md{"template"} . " does not exist\n";
	}
	# Note: if different templates are specified for different languages,
	# the one from the last language analyzed is used.
	# FIXME: change this - if different templates are specified for different langs,
	# the template isn't checked for existance in other langs
	# Wait, why not just use the actual template given? It's stored in $config->{"metadata"} anyways
	$config->{"metadata"}->{$pageid}->{"template"} = $tmp_md{"template"};
	foreach my $md_id (split / /, $config->{"templates"}->{$tmp_md{"template"} . ".$lang.html"}->{"metadata"}) {
		if (!exists($tmp_md{$md_id})) {
			die "ERROR: $fullname does not include \"$md_id\" metadata\n";
		}
	}
	foreach my $md_id (keys %tmp_md) {
		$config->{"metadata"}->{$pageid}->{$lang}->{$md_id} = $tmp_md{$md_id};
	}
	$config->{"metadata"}->{$pageid}->{"dirname"} = $dirname;
	$config->{"metadata"}->{$pageid}->{"basename"} = $basename;
}

sub gen_metadata_hash {
	find(\&parse_metadata, "pages/");
}

sub check_metadata_langs {
	my $not_found;
	foreach my $pageid (keys %{$config->{"metadata"}}) {
		$not_found = "";
		foreach my $lang (keys %{$config->{"langs"}}) {
			if (!exists($config->{"metadata"}->{$pageid}->{$lang})) {
				$not_found .= " $lang";
			}
		}
		if ($not_found) {
			die("ERROR: languages \"$not_found\" not found for $pageid\n");
		}
	}
}

sub init_metadata {
	gen_metadata_hash();
	check_metadata_langs();
}

1;
